# -*- coding: utf-8 -*-
"""

Reading and plotting Mismip loop results
Statistical moments and box plots
All ensembles
v2

"""


import os
import numpy as np
import netCDF4 as nc
import matplotlib.pyplot as plt
from scipy import stats
from scipy.stats import shapiro

nruns           = 500 #nb of ensemble members
nsteps          = 500 #nb of time steps

periodChangConv = 50.0 #period at end of simulation to evaluate convergence of mean and standard deviation [yr

DirResults         = os.path.dirname(os.path.realpath(__file__))+'/'
controlPath        = DirResults+'ControlTransientConstMeltDw1_id1TimeSeries_v1.csv'

rholevels  = np.array([0.0,0.6,0.9,0.95,0.98])
pathsRes   = [DirResults+'NCfileMISMIP_StochMelt_547Ens_v1.nc',
              DirResults+'NCfileMISMIP_RunARonlyMeltAc6_546Ens_v1.nc',
              DirResults+'NCfileMISMIP_RunARonlyMeltAc9_547Ens_v1.nc',
              DirResults+'NCfileMISMIP_RunARonlyMeltAc95_546Ens_v1.nc',
              DirResults+'NCfileMISMIP_RunARonlyMeltAc98_546Ens_v1.nc']
taulevels  = np.round(1/(1-rholevels))

ls_labels = ['$MISMIP_{\u03C4=1}$',
             '$MISMIP_{\u03C4=2.5}$',
             '$MISMIP_{\u03C4=10}$',
             '$MISMIP_{\u03C4=20}$',
             '$MISMIP_{\u03C4=50}$']

# Load control run #
ctrl       = np.genfromtxt(controlPath,dtype=float,delimiter=',')
vctrl      = ctrl[1,:]
mctrl      = 917*1e-12*vctrl

ls_colors   = plt.cm.rainbow(np.linspace(0,1,len(taulevels)))


for ii,tauval in enumerate(taulevels):
    ds = nc.Dataset(pathsRes[ii])
    if(ii==0):
        ### Load ###
        timear     = np.array(ds.variables['time'])
        matMvals   = 917*1e-12*np.array(ds.variables['volume'])[0:nruns,:] #[Gt]
        ### Stats ###
        MnMvals     = np.mean(matMvals,axis=0)
        SdevMvals   = np.std(matMvals,axis=0)
        SkewMvals   = stats.skew(matMvals,axis=0)
        KurtMvals   = stats.kurtosis(matMvals,axis=0)
    else:
        ### Load ###
        matMvals   = np.row_stack((matMvals,917*1e-12*np.array(ds.variables['volume'])[0:nruns,:])) #[Gt]
        ### Stats ###
        MnMvals     = np.row_stack((MnMvals,np.mean(matMvals[-1*nruns:,:],axis=0)))
        SdevMvals   = np.row_stack((SdevMvals,np.std(matMvals[-1*nruns:,:],axis=0)))
        SkewMvals   = np.row_stack((SkewMvals,stats.skew(matMvals[-1*nruns:,:],axis=0)))
        KurtMvals   = np.row_stack((KurtMvals,stats.kurtosis(matMvals[-1*nruns:,:],axis=0)))
    ds.close()

### Stats plots ###
plotvals  = np.copy(matMvals)
plotmeans = np.copy(MnMvals)
plotsdevs = np.copy(SdevMvals)
plotskews = np.copy(SkewMvals)
plotkurts = np.copy(KurtMvals)

### Plotting ###
fig = plt.figure(figsize=[11,10.5])
ax  = plt.subplot(311)
for rr in range(len(taulevels)):
    ax.plot(timear,plotsdevs[rr,:],c=ls_colors[rr],linewidth=2.0,label=ls_labels[rr])
ax.tick_params(axis='y', which='major', labelsize=15)
ax.set_ylabel(r'$\sigma_{E}$ in ice mass [Gt]',fontsize=15) 
ax.tick_params(axis='y', which='major', labelsize=14)   
ax.tick_params(axis='x', which='major', labelcolor='w')   
ax.legend(loc='upper left',fontsize=13)
ax.text(-75,1550,'(a)',fontsize=14)
ax  = plt.subplot(312)
for rr in range(len(taulevels)):
    ax.plot(timear,plotskews[rr,:],c=ls_colors[rr],linewidth=2.0,label=ls_labels[rr])
ax.tick_params(axis='both', which='major', labelsize=15)
ax.set_xlabel('Years',fontsize=15)
ax.set_ylabel('Skewness',fontsize=15)   
ax.text(-75,0.45,'(b)',fontsize=14)
ax  = plt.subplot(313)
boxplotdata = [plotvals[kk*nruns:(kk+1)*nruns,-1]-mctrl[0] for kk in range(0,len(pathsRes))]
boxplotdata = boxplotdata[::-1]
bp = ax.boxplot(boxplotdata,
                medianprops=dict(color='b',linestyle='-'),
                flierprops=dict(markerfacecolor='r',marker='o'),
                vert=0)
ax.tick_params(axis='x', which='major', labelsize=14)
ax.set_yticks([kk for kk in range(1,1+len(ls_labels))],ls_labels[::-1])
ax.tick_params(axis='both', which='major', labelsize=14)
ax.set_xlabel(r'Ice Mass Change [Gt]',fontsize=14)
ax.set_xlim([-1.03*np.max(abs(np.array(boxplotdata))),1.03*np.max(abs(np.array(boxplotdata)))])
ax.text(-5900,5.5,'(c)',fontsize=14)
ax.tick_params(axis='x', which='major', labelsize=14)
fig.tight_layout()

### Print Stats ###
i0 = np.where(timear>=timear[-1]-periodChangConv)[0][0]
for rr in range(len(taulevels)):
    ShapiroWilk_pValue = shapiro(plotvals[rr*nruns:(rr+1)*nruns,-1])[1]
    linregrMn          = stats.linregress(timear[i0:],plotmeans[rr,i0:])
    trendMn            = linregrMn.slope
    pvaltrendMn        = linregrMn.pvalue
    linregrSdev        = stats.linregress(timear[i0:],plotsdevs[rr,i0:])
    trendSdev          = linregrSdev.slope
    pvaltrendSdev      = linregrSdev.pvalue
    print(f'Results from {ls_labels[rr]}:')
    print(f'Mean: {np.round(plotmeans[rr,-1],4)}')
    print(f'Sdev: {np.round(plotsdevs[rr,-1],4)}')
    print(f'Skew: {np.round(plotskews[rr,-1],4)}')
    print(f'p-value in Shapiro-Wilk normality test: {np.round(ShapiroWilk_pValue,3)}')
    print(f'Trend in Mean over the last {periodChangConv} yr: {np.round(trendMn,2)} (p-value: {np.round(pvaltrendMn,4)})')
    print(f'Trend in Std Dev over the last {periodChangConv} yr: {np.round(trendSdev,2)} (p-value: {np.round(pvaltrendSdev,4)})')
    print(f'Mean total change: {np.round(plotmeans[rr,-1]-mctrl[0],1)}')
    print(f'Mean relative change: {100*np.round((plotmeans[rr,-1]-mctrl[0])/mctrl[0],4)} %')
    print('')
    print('')
print(f'Ratios of the standard deviations: {np.round(plotsdevs[:,-1]/plotsdevs[0,-1],3)}')  